//+------------------------------------------------------------------+
//|                                                       OsMACD.mq5 |
//|                                            Copyright 2019, ernst |
//|                             https://www.mql5.com/en/users/pippod |
//+------------------------------------------------------------------+
#property copyright "Copyright 2019, ernst"
#property link      "https://www.mql5.com/en/users/pippod"
#property version   "1.00"
#property indicator_separate_window
#property indicator_buffers 7
#property indicator_plots   3
//--- plot MACD
#property indicator_label1  "MACD"
#property indicator_type1   DRAW_LINE
#property indicator_color1  clrAqua
#property indicator_style1  STYLE_SOLID
#property indicator_width1  1
//--- plot Signal
#property indicator_label2  "Signal"
#property indicator_type2   DRAW_LINE
#property indicator_color2  clrMagenta
#property indicator_style2  STYLE_SOLID
#property indicator_width2  1
//--- plot OsMA
#property indicator_label3  "OsMA"
#property indicator_type3   DRAW_COLOR_HISTOGRAM
#property indicator_color3  clrRoyalBlue,clrFireBrick,clrYellow
#property indicator_style3  STYLE_SOLID
#property indicator_width3  5
//--- input parameters
input ENUM_APPLIED_PRICE Price=PRICE_TYPICAL;
input int      FastMAPeriod=12;
input int      SlowMAPeriod=26;
input ENUM_MA_METHOD MAMethod=MODE_LWMA;
input int      SignalPeriod=9;
input ENUM_MA_METHOD SignalMethod=MODE_SMA;
input ENUM_TIMEFRAMES Timeframe=PERIOD_H1;
//--- indicator buffers
double         PriceBuffer[];
double         OsMABuffer[];
double         ColorBuffer[];
double         FastMABuffer[];
double         SlowMABuffer[];
double         MacdBuffer[];
double         SignalBuffer[];

#include <MovingAverages.mqh>
int weightA,weightB,weightC;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   IndicatorSetString(INDICATOR_SHORTNAME,StringFormat("OsMACD %s(%d,%d,%d)",StringSubstr(EnumToString(MAMethod),5),FastMAPeriod,SlowMAPeriod,SignalPeriod));
//--- indicator buffers mapping
   SetIndexBuffer(0,MacdBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,SignalBuffer,INDICATOR_DATA);
   SetIndexBuffer(2,OsMABuffer,INDICATOR_DATA);
   SetIndexBuffer(3,ColorBuffer,INDICATOR_COLOR_INDEX);
   SetIndexBuffer(4,FastMABuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(5,SlowMABuffer,INDICATOR_CALCULATIONS);
   SetIndexBuffer(6,PriceBuffer,INDICATOR_CALCULATIONS);
//---
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,SlowMAPeriod+SignalPeriod);
//---
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---
   Price(rates_total,prev_calculated,open,high,low,close,PriceBuffer);
//---
   switch(MAMethod)
     {
      case MODE_SMA:
         SimpleMAOnBuffer(rates_total,prev_calculated,0,FastMAPeriod,PriceBuffer,FastMABuffer);
         SimpleMAOnBuffer(rates_total,prev_calculated,0,SlowMAPeriod,PriceBuffer,SlowMABuffer);
         break;
      case MODE_EMA:
         ExponentialMAOnBuffer(rates_total,prev_calculated,0,FastMAPeriod,PriceBuffer,FastMABuffer);
         ExponentialMAOnBuffer(rates_total,prev_calculated,0,SlowMAPeriod,PriceBuffer,SlowMABuffer);
         break;
      case MODE_SMMA:
         SmoothedMAOnBuffer(rates_total,prev_calculated,0,FastMAPeriod,PriceBuffer,FastMABuffer);
         SmoothedMAOnBuffer(rates_total,prev_calculated,0,SlowMAPeriod,PriceBuffer,SlowMABuffer);
         break;
      case MODE_LWMA:
         LinearWeightedMAOnBuffer(rates_total,prev_calculated,0,FastMAPeriod,PriceBuffer,FastMABuffer,weightA);
         LinearWeightedMAOnBuffer(rates_total,prev_calculated,0,SlowMAPeriod,PriceBuffer,SlowMABuffer,weightB);
     }
//---
   int limit=!prev_calculated?FastMAPeriod:prev_calculated-1;
   for(int i=limit;i<rates_total && !_StopFlag;i++)
      MacdBuffer[i]=FastMABuffer[i]-SlowMABuffer[i];
//---
   switch(SignalMethod)
     {
      case MODE_SMA:
         SimpleMAOnBuffer(rates_total,prev_calculated,SlowMAPeriod,SignalPeriod,MacdBuffer,SignalBuffer);
         break;
      case MODE_EMA:
         ExponentialMAOnBuffer(rates_total,prev_calculated,SlowMAPeriod,SignalPeriod,MacdBuffer,SignalBuffer);
         break;
      case MODE_SMMA:
         SmoothedMAOnBuffer(rates_total,prev_calculated,SlowMAPeriod,SignalPeriod,MacdBuffer,SignalBuffer);
         break;
      case MODE_LWMA:
         LinearWeightedMAOnBuffer(rates_total,prev_calculated,SlowMAPeriod,SignalPeriod,MacdBuffer,SignalBuffer,weightC);
     }
//---
   limit=!prev_calculated?SlowMAPeriod+SignalPeriod-1:prev_calculated-1;
   for(int i=limit;i<rates_total && !_StopFlag;i++)
     { 
      OsMABuffer[i]=MacdBuffer[i]-SignalBuffer[i];
      ColorBuffer[i]=OsMABuffer[i]>OsMABuffer[i-1]?0:OsMABuffer[i]<OsMABuffer[i-1]?1:2;
     }
//--- return value of prev_calculated for next call
   return(rates_total);
  }
//+------------------------------------------------------------------+
//| Price applied                                                    |
//+------------------------------------------------------------------+
int Price(const int rates_total,
          const int prev_calculated,
          const double &open[],
          const double &high[],
          const double &low[],
          const double &close[],
          double &price[])
  {
//---bars to count
   int begin=(rates_total!=prev_calculated)?prev_calculated:prev_calculated-1;
//---price applied
   switch(Price)
     {
      case PRICE_CLOSE:
         for(int i=begin;i<rates_total && !_StopFlag;i++)
            price[i]=close[i];
         break;
      case PRICE_OPEN:
         for(int i=begin;i<rates_total && !_StopFlag;i++)
            price[i]=open[i];
         break;
      case PRICE_HIGH:
         for(int i=begin;i<rates_total && !_StopFlag;i++)
            price[i]=high[i];
         break;
      case PRICE_LOW:
         for(int i=begin;i<rates_total && !_StopFlag;i++)
            price[i]=low[i];
         break;
      case PRICE_MEDIAN:
         for(int i=begin;i<rates_total && !_StopFlag;i++)
            price[i]=(high[i]+low[i])/2;
         break;
      case PRICE_TYPICAL:
         for(int i=begin;i<rates_total && !_StopFlag;i++)
            price[i]=(high[i]+low[i]+close[i])/3;
         break;
      case PRICE_WEIGHTED:
         for(int i=begin;i<rates_total && !_StopFlag;i++)
            price[i]=(high[i]+low[i]+close[i]+close[i])/4;
         break;
      default:
         for(int i=begin;i<rates_total && !_StopFlag;i++)
            price[i]=close[i];
     }
//---
   return(rates_total);
  }
//+------------------------------------------------------------------+
